@@ -15,7 +15,7 @@ my %params;
 GetOptions(
     \%params,
     "bootloader-probe",    # Prints the bootloader in use on the system
-    "arch-probe",          # Prints the arch of the system
+    "arch-probe:s",        # Prints the arch of the system
     "bootloader|b=s",
     "config_file=s",
     "add-kernel|a=s",
@@ -38,6 +38,10 @@ GetOptions(
     "default",
     "help",
     "man",
+    "xen",
+    "xenhyper|xh=s",
+    "xenhyper-args|xha=s",
+    "update-xenhyper=s",
 );
 
 &usage if ( !%params || defined $params{help} );
@@ -50,13 +54,11 @@ sub detect_architecture {
 
     my $arch;
     if ( $arch_style eq 'linux' ) {
-        $arch = `uname -m | sed -e s/i.86/i386/ -e s/sun4u/sparc64/ -e s/arm.*/arm/ \
-	-e s/sa110/arm/ -e s/s390x/s390/ -e s/parisc64/parisc/`;
+        $arch = `uname -m | sed -e s/i.86/i386/ -e s/sun4u/sparc64/ -e s/arm.*/arm/ -e s/sa110/arm/ -e s/s390x/s390/ -e s/parisc64/parisc/`;
         chomp $arch;
     }
     elsif ( $arch_style eq 'gentoo' ) {
-        $arch = `uname -m | sed -e s/i.86/x86/ -e s/sun4u/sparc/ -e s/arm.*/arm/ \
-	-e s/sa110/arm/ -e s/x86_64/amd64/ -e s/sparc.*/sparc/ -e s/parisc.*/hppa/`;
+        $arch = `uname -m | sed -e s/i.86/x86/ -e s/sun4u/sparc/ -e s/arm.*/arm/ -e s/sa110/arm/ -e s/x86_64/amd64/ -e s/sparc.*/sparc/ -e s/parisc.*/hppa/`;
         chomp $arch;
     }
     else {
@@ -163,6 +165,7 @@ sub _info_grub {
         boot        => '^\s*root\s+(.*)',
         initrd      => '^\s*initrd\s+(.*)',
         savedefault => '^\s*savedefault\s+(.*)',
+        module      => '^\s*module\s+(.+)',
     );
 
     my @sections;
@@ -174,6 +177,7 @@ sub _info_grub {
         }
         foreach my $key ( keys %matches ) {
             if ( $_ =~ /$matches{$key}/i ) {
+                $key .= '2' if exists $sections[$index]{$key};
                 $sections[$index]{$key} = $1;
                 if ( $key eq 'args' ) {
                     $sections[$index]{$key} =~ s/root=\S+\s*//i;
@@ -312,12 +316,30 @@ sub add_grub {
     push( @newkernel, "\troot $param{boot}\n" )  if defined $param{boot};
 
     my $line;
-    $line = "\tkernel $param{'add-kernel'}" if defined $param{'add-kernel'};
-    $line = $line . " root=$param{root}"    if defined $param{root};
-    $line = $line . " $param{args}"         if defined $param{args};
-    push( @newkernel, "$line\n" );
+    if ( defined $param{xen} ) {
+        if ( defined $param{'xenhyper'} ) {
+            $line = "\tkernel $param{'xenhyper'}";
+            $line .= " $param{'xenhyper-args'}" if defined $param{'xenhyper-args'};
+            push ( @newkernel, "$line\n" );
+        } else {
+            $line = "\tkernel $sections[$default]{kernel}";
+            $line .= " $sections[$default]{root}" if defined $sections[$default]{root};
+            $line .= " $sections[$default]{args}" if defined $sections[$default]{args};
+            push( @newkernel, "$line\n" );
+        }
+        $line = "\tmodule $param{'add-kernel'}" if defined $param{'add-kernel'};
+        $line .= " root=$param{root}"    if defined $param{root};
+        $line .= " $param{args}"         if defined $param{args};
+        push( @newkernel, "$line\n" );
+        push( @newkernel, "\tmodule $param{initrd}\n" ) if defined $param{initrd};
+    } else {
+        $line = "\tkernel $param{'add-kernel'}" if defined $param{'add-kernel'};
+        $line .= " root=$param{root}"    if defined $param{root};
+        $line .= " $param{args}"         if defined $param{args};
+        push( @newkernel, "$line\n" );
+        push( @newkernel, "\tinitrd $param{initrd}\n" ) if defined $param{initrd};
+    }
 
-    push( @newkernel, "\tinitrd $param{initrd}\n" ) if defined $param{initrd};
     push( @newkernel, "\tsavedefault $param{savedefault}\n" )
       if defined $param{savedefault};
     push( @newkernel, "\n" );
@@ -363,8 +385,9 @@ sub update_grub {
 
     print("Updating kernel.\n") if &debug() > 1;
 
-    if ( !defined $params{'update-kernel'}
-        || ( !defined $params{'args'} && !defined $params{'remove-args'} ) )
+    if ( !defined $params{'update-kernel'} && !defined $params{'xenhyper-args'}
+        || ( !defined $params{'args'} && !defined $params{'remove-args'} 
+        && !defined $params{'xenhyper-args'} ) )
     {
         warn
 "ERROR:  kernel position or title (--update-kernel) and args (--args or --remove-args) required.\n";
@@ -390,19 +413,30 @@ sub update_grub {
         return undef;
     }
 
+    my $kregex = '(^\s*kernel\s+\S+)(.*)';
+    if ( !defined $params{'update-xenhyper'} && defined $params{'xen'} )
+    {
+    $kregex = '(^\s*module\s+\S+vmlinuz\S+)(.*)' if defined $params{'xen'};
+    }
+
     my $index = -1;
     foreach (@config) {
         if ( $_ =~ /^\s*title/i ) {
             $index++;
         }
         if ( $index == $params{'update-kernel'} ) {
-            if ( $_ =~ /(^\s*kernel\s+\S+\s+)(.*)\n/i ) {
+            if ( $_ =~ /$kregex/i ) {
                 my $kernel = $1;
                 my $args   = $2;
-                $args =~ s/\s+$params{'remove-args'}\=*\S*//ig
+                $args =~ s/\s+$params{'remove-args'}(\=\S+|\s+|$)/ /ig
                   if defined $params{'remove-args'};
-                $args = $args . " " . $params{'args'}
-                  if defined $params{'args'};
+                if ( defined $params{'args'} || defined $params{'xenhyper-args'} ) {
+                    $params{'args'} = $params{'xenhyper-args'} if defined $params{'xenhyper-args'};
+                    my $base_arg = $params{'args'};
+                    $base_arg =~ s/\=.*//;
+                    $args =~ s/\s+$base_arg(\=\S+|\s+|$)/ /ig;
+                    $args = $args . " " . $params{'args'};
+                }
                 if ( $_ eq $kernel . $args . "\n" ) {
                     warn "WARNING:  No change made to args.\n";
                     return undef;
@@ -968,7 +1002,7 @@ sub print_info {
     for my $index ( $start .. $end ) {
         print "\nindex\t: $index\n";
         $index++;
-        foreach ( keys( %{ $sections[$index] } ) ) {
+        foreach ( sort keys( %{ $sections[$index] } ) ) {
             print "$_\t: $sections[$index]{$_}\n";
         }
     }
@@ -1034,7 +1068,7 @@ if ( defined $params{'bootloader-probe'} ) {
     exit 0;
 }
 elsif ( defined $params{'arch-probe'} ) {
-    $detected_architecture = detect_architecture()
+    $detected_architecture = detect_architecture( $params{'arch-probe'} )
       || warn "Could not detect architecture\n";
     print "$detected_architecture\n";
     exit 0;
@@ -1063,7 +1097,7 @@ $params{config_file} = $cfg_files{$bootloader}
 die("Can't read config file.\n") unless ( -r $params{config_file} );
 &debug( $params{'debug'} ) if ( defined $params{'debug'} );
 
-if ( defined $params{'add-kernel'} ) {
+if ( defined $params{'add-kernel'} || defined $params{'xenhyper'} ) {
     &read( $params{config_file} );
     &add(%params);
     &write( $params{config_file} );
@@ -1077,7 +1111,8 @@ elsif ( defined $params{'remove-kernel'} ) {
     &install() unless $detected_bootloader eq 'grub';
 
 }
-elsif ( defined $params{'update-kernel'} ) {
+elsif ( defined $params{'update-kernel'} || defined $params{'update-xenhyper'} ) {
+    $params{'update-kernel'} = $params{'update-xenhyper'} if defined $params{'update-xenhyper'};
     &read( $params{config_file} );
     &update(%params);
     &write( $params{config_file} );
@@ -1126,12 +1161,16 @@ boottool [--bootloader-probe] [--arch-probe]
          [--remove-kernel=<#|title|start|end>]
          [--update-kernel=<#|title>] [--remove-args=<args>]
          [--info=<all|default|#>] [--default] [--set-default=<#>]
+         [--xen] [--xenhyper=<kernel_path>] [--xenhyper-args=<kernel_args>]
          [--help] [--debug=<0..5>]
 
 Examples:
     boottool --info all					# Print config info
     boottool -a /boot/vmlinuz -t 'test' -p end		# Add a new kernel
     boottool --remove-kernel 3				# Remove a kernel
+    boottool -u title1 -remove-args 'ro'		# Remove args from a kernel
+    boottool -u title1 -args 'arg1=test' --xen		# Add args to a Xen kernel
+    boottool -xh /boot/xen.gz -a /boot/vmlinuz -t test --xen 	# Add Xen kernel
     boottool --set-default 1				# Set new default\n";
     exit 1;
 }
