@@ -68,40 +68,45 @@ use base 'Linux::Bootloader';
 
 
 use vars qw( $VERSION );
-our $VERSION = '1.1';
+our $VERSION = '1.2';
 
 
 sub new {
-    my $class = shift;
+    my $this = shift;
+    my $class = ref($this) || $this;
     my $self = bless({}, $class);
 
-    $self->{'config'}   = [];
-    $self->{'debug'}    = 0;
-
     $self->SUPER::new();
 
     return $self;
 }
 
 
+sub _set_config_file {
+    my $self=shift;
+    $self->{'config_file'}='/etc/yaboot.conf';
+}
+
 ### YABOOT functions ###
 
 
 # Run command to install bootloader
 
 sub install {
-  my $self=shift;
+    my $self=shift;
 
-  system("ybin");
-  if ($? != 0) { 
-    warn ("ERROR:  Failed to run ybin.\n") && return undef; 
-  }
-  return 1;
+    #system("/usr/sbin/ybin");
+    #if ( $? != 0 ) {
+    #    warn("ERROR:  Failed to run ybin.\n") && return undef;
+    #}
+
+    print("Not installing bootloader.\n");
+    print("Depending on your arch you may need to run ybin.\n");
+    return 1;
 }
 
 
 1;
-__END__
 
 
 =head1 AUTHOR
@@ -120,5 +125,5 @@ under the same terms as Perl itself.
 
 L<Linux::Bootloader>
 
-=end
+=cut
 
